# frozen_string_literal: true

require 'oj'
require_relative '../cache/redis_cache.rb'

module EodFacade
  class Options < ::EodFacade::Base
    class << self
      def call(symbol)
        underlying = EodServices::Contract.underlying_symbol(symbol)
        expiry = EodServices::Contract.contract_expiry(symbol)
        option_type = contract_type(symbol)

        unless expiry
          raise ArgumentError, "Invalid expiration date for option #{symbol}"
        end

        unless Cache::RedisCache.get(redis_key(underlying, expiry))
          response = make_request(url_path(underlying))
          
          unless response.success?
            raise ArgumentError, "Error fetching options data for #{underlying}"
          end

          Cache::RedisCache.set(
            redis_key(underlying, expiry),
            response.parsed_response.to_json
          )
        end

        contracts = Oj.load(Cache::RedisCache.get(redis_key(underlying, expiry)))

        contract_hash(
          symbol: symbol,
          contracts: contracts,
          option_type: option_type
        )
      end

      private

      def params(expiry)
        {
          from: expiry.to_s,
          to: expiry.to_s
        }
      end

      def url_path(underlying)
        "/options/#{underlying}"
      end

      def contract_hash(symbol:, contracts:, option_type:)
        by_expiration = contracts['data'].first
        expiration_date = by_expiration && by_expiration['expirationDate']

        by_option_type = by_expiration && by_expiration['options'][option_type]
        contract = by_option_type&.find do |c|
          c['contractName'] == symbol
        end

        raise ArgumentError, "Contract #{symbol} not found" unless contract

        contract.merge('expirationDate' => expiration_date)
      end

      def contract_type(symbol)
        option_type = EodServices::Contract.contract_type(symbol)
        case option_type
        when EodModels::OptionType::CALL
          'CALL'
        when EodModels::OptionType::PUT
          'PUT'
        else
          'STOCK'
        end
      end

      def redis_key(underlying, expiry)
        "#{underlying}_#{expiry.to_s}_options"
      end
    end
  end
end
